from trytond.model import ModelView, ModelSQL, fields, Unique, UnionMixin
from trytond.pool import Pool
from trytond.exceptions import UserError
from trytond.i18n import gettext
from trytond.transaction import Transaction
from trytond.pyson import If, Eval, Bool
from datetime import datetime, date, timedelta

UTILIZATION_ANIMAL =  [
        ('meat', 'Meat'),
        ('milk', 'Milk'),
        ('draught', 'Draught'),
        ('pack_animal', 'Pack Animal'),
        ('hide', 'Hide'),
        ('riding', 'Riding'),
        ('fleece', 'Fleece'),
        ]

ANIMAL_TYPE = [
        ('horse', 'Horse'),
        ('donkey', 'Donkey'),
        ('cattle', 'Cattle'),
        ('sheep', 'Sheep'),
        ('goat', 'Goat'),
        ('pig', 'Pig'),
        ('rabbit', 'Rabbit'),
        ]

SEX = [
        ('femele', 'Femele'),
        ('male', 'Male'),
        ]

CYCLE_LIFE = [
        ('calf' , 'Calf'),
        ('heifer', 'Heifer'),
        ('lactating_cow', 'Lactating Cow'),
        ('parturition', 'Parturition'),
        ('drying-off', 'Drying-off'),
        ('dry_period', 'Dry Period'),

        ]

_TYPE = [
    ('milk', 'Milk'),
    ('meat', 'Meat'),
]

STATES = [
        ('calf' , 'Calf'),
        ('heifer', 'Heifer'),
        ('lactating_cow', 'Lactating Cow'),
        ('parturition', 'Parturition'),
        ('drying-off', 'Drying-off'),
        ('dry_period', 'Dry Period'),

        ]



class MultiFarm(ModelSQL, ModelView):
    "MultiFarm"
    __name__= 'multifarm.multifarm'

    tipos = fields.Char('Tipos', required=True)


class Utilization(ModelSQL, ModelView):
    "Utilization"
    __name__ = 'multifarm.utilization'

    type = fields.Selection(UTILIZATION_ANIMAL, 'Type', required=True)

    @classmethod
    def __setup__(cls):
        super(Utilization, cls).__setup__()
        t=cls.__table__()
        cls._sql_constraints += [
                ('type_uniq', Unique(t, t.type),
                    'multifarm.utilization_type_unique'),
                ]


class Specie(ModelSQL, ModelView):
    "Specie"
    __name__ = 'multifarm.specie'

    name = fields.Selection(ANIMAL_TYPE, 'Name', required=True)
    breed = fields.One2Many('multifarm.breed','specie', 'Breed')

    @classmethod
    def __setup__(cls):
        super(Specie, cls).__setup__()
        t=cls.__table__()
        cls._sql_constraints += [
                ('name_uniq', Unique(t, t.name),
                    'multifarm.specie_name_unique'),
                ]


class Breed(ModelSQL, ModelView):
    "Breed"
    __name__ = 'multifarm.breed'

    name = fields.Char('Breed')
    specie  = fields.Many2One('multifarm.specie',  'Specie' )
    origin = fields.Char('Origin')
    average_weight_femele = fields.Integer('Average Weight Femele')
    average_weight_male = fields.Integer('Average Weight Male')

    @classmethod
    def __setup__(cls):
        super(Breed, cls).__setup__()
        t=cls.__table__()
        cls._sql_constraints += [
                ('name_uniq', Unique(t, t.name),
                    'multifarm.breed_name_unique'),
                ]






class Animals(ModelSQL, ModelView):
    "Animals"
    __name__ =  'multifarm.animals'

    _rec_name = 'number'

    number = fields.Integer('Number', required=True)
    nickname = fields.Char('Nickname')
    specie = fields.Function(fields.Char('Specie'), 'on_change_with_specie')
    breed = fields.Many2One('multifarm.breed', 'Breed', required=True)
    sex = fields.Selection(SEX, 'Sex', required=True)
    last_weight = fields.Function(fields.Integer('Last Weight'), 'last_weights_weight')
    last_weight_datetime = fields.Function(fields.DateTime('Last Weight DateTime'), 'last_weights_datetime') 
    weights = fields.One2Many('multifarm.weights', 'animal', 'Weights')
    group = fields.Many2One('multifarm.groups', 'Group')
    birth = fields.Date('Birth')
    age = fields.Function(fields.Integer('Age'), 'getter_age')
    state = fields.Selection(STATES, 'State')
    type = fields.Selection(_TYPE, 'Type', select=True,
            required=True, states={
                'readonly': ((Eval('state') != 'draft')
                    | Eval('context', {}).get('type')
                    | (Eval('lines', [0]) & Eval('type'))),
                }, depends=['state'])
    

    @fields.depends('breed')
    def on_change_with_specie(self, name=None):
        if self.breed:
            if self.breed.specie.name:
                return self.breed.specie.name
            else:
                return "Tostada con Café"

    @classmethod
    def __setup__(cls):
        super(Animals, cls).__setup__()
        t=cls.__table__()
        cls._sql_constraints += [
                ('number_uniq', Unique(t, t.number),
                    'multifarm.animals_number_unique'),
                ]

    def getter_age(self, name):
        if not self.birth:
            return None 
        age = date.today() - self.birth
        #raise UserError(str(age.days))
        return age.days

    @fields.depends('weights')
    def last_weights_datetime(self, name):
        if self.weights:
            #raise UserError(str(max(self.weights).name))
            return max(self.weights).weight_date_time

    @fields.depends('weights')
    def last_weights_weight(self, name):
        if self.weights:
            return max(self.weights).weight

class Groups(ModelSQL, ModelView):
    "Groups"
    __name__ = 'multifarm.groups'

    name = fields.Char('Group', required=True)
    animals = fields.One2Many('multifarm.animals', 'group', 'Animals')

class Weights(ModelSQL, ModelView):
    "Weights"
    __name__ = 'multifarm.weights'

    _rec_name = 'weight'

    animal = fields.Many2One('multifarm.animals', 'Animal', required=True)
    weight = fields.Integer('Weight')
    weight_date_time = fields.DateTime('Date_Time')

class WeightsExpected(ModelSQL, ModelView):
    "Weights Expected"

    __name__ = 'multifarm.weights_expected'

    animal = fields.Many2One('multifarm.animals', 'Animal', required=True)
    weight_third_month = fields.Function(fields.Integer('Weight third Month'), 'on_change_with_weight_third_month')


    @fields.depends('weightsdsdsd')
    def on_change_with_weight_third_month(self, name=None):
        raise UserError(str(9))
        if self.weights:
            raise UserError(str(STATES))
        else:
            raise UserError(str(STATES))

